@@ -35,6 +35,8 @@ class ImageIOMixin:
         :type filename: str
         :param alpha: include alpha plane if present, defaults to False
         :type alpha: bool, optional
+        :param rgb: force color image to be in RGB order, defaults to True
+        :type rgb: bool, optional
         :param kwargs: options applied to image frames, see :func:`~machinevisiontoolbox.base.imageio.convert`
         :raises ValueError: file not found
         :return: image from file
@@ -56,7 +58,7 @@ class ImageIOMixin:
             >>> Image.Read('flowers1.png', reduce=4)
             >>> Image.Read('flowers1.png', gamma='sRGB') # linear tristimulus values
 
-        .. note::  If the path is not absolute it is first searched for relative
+        :note:  If the path is not absolute it is first searched for relative
             to the current directory, and if not found, it is searched for in
             the ``images`` folder of the ``mvtb_data`` package.
 
@@ -88,7 +90,7 @@ class ImageIOMixin:
 
         :param title: named of window, defaults to image ``name``
         :type title: bool
-        :param kwargs: options, see :func:`~machinevisiontoolbox.base.idisp`
+        :param kwargs: options, see :func:`~machinevisiontoolbox.base.imageio.idisp`
 
         Display an image using either Matplotlib (default) or OpenCV.
 
@@ -114,8 +116,7 @@ class ImageIOMixin:
 
         return idisp(self.A,
                 title=title,
-                bgr=self.isbgr,
-                colororder=self.colororder_str,
+                colororder="RGB" if self.isrgb else "BGR",
                 **kwargs)
 
     def write(self, filename, dtype='uint8', **kwargs):
@@ -175,7 +176,7 @@ class ImageIOMixin:
             >>> meta['FocalLength']
             >>> img.metadata('FocalLength')  # get specific metadata item
 
-        .. note::  Metadata items will be converted, where possible, to int or float values.
+        :note:  Metadata items will be converted, where possible, to int or float values.
         
         """
         try:
@@ -429,7 +430,7 @@ class ImageIOMixin:
             >>> right = Image.Read("rocks2-r.png", reduce=2)
             >>> left.stdisp(right)
 
-        .. note:: The images are assumed to be epipolar aligned.
+        :note: The images are assumed to be epipolar aligned.
 
         :reference:
             - Robotics, Vision & Control for Python, Section 14.4, P. Corke, Springer 2023.
@@ -522,12 +523,21 @@ if __name__ == "__main__":
     import pathlib
     import os.path
 
+    from machinevisiontoolbox import VideoCamera
+    import time
+    camera = VideoCamera(1)
+    time.sleep(10)
+    for i in range(10):
+        image = camera.grab()
+        time.sleep(0.1)
+
+
+    camera.release()
+    image.disp()
+
     from machinevisiontoolbox import *
     church = Image.Read('shark2.png')
     print(church.metadata())
     church.disp(block=True)
-
-
-
     
     # exec(open(pathlib.Path(__file__).parent.parent.absolute() / "tests" / "test_processing.py").read())  # pylint: disable=exec-used
\ No newline at end of file
