@@ -2,6 +2,7 @@ import numpy as np
 import urllib.request
 from pathlib import Path
 import warnings
+import time
 
 import cv2 as cv
 import copy
@@ -18,10 +19,13 @@ from machinevisiontoolbox.base.data import mvtb_path_to_datafile
 #import pyautogui  # requires pip install pyautogui
 from spatialmath.base import islistof
 
+__last_windowname = None
+
 def idisp(im,
-          bgr=False,
+          colororder="RGB",
           matplotlib=True,
           block=False,
+          fps=None,
 
           fig=None,
           ax=None,
@@ -55,8 +59,8 @@ def idisp(im,
           ynormal=False,
           extent=None,
 
+          coordformat=None,
           savefigname=None,
-          colororder="RGB",
           **kwargs):
 
     """
@@ -64,12 +68,14 @@ def idisp(im,
 
     :param im: image to display
     :type im: ndarray(H,W), ndarray(H,W,3)
-    :param bgr: image is in BGR (native OpenCV color) order, defaults to False
-    :type bgr: bool, optional
+    :param colororder: color order, defaults to "RGB"
+    :type colororder: str
     :param matplotlib: plot using Matplotlib (True) or OpenCV (False), defaults to True
     :type matplotlib: bool, optional
-    :param block: Matplotlib figure blocks until window closed, defaults to False
-    :type block: bool, optional
+    :param block: after display, Matplotlib blocks until window closed or for the specified time period, defaults to False
+    :type block: bool, float, optional
+    :param fps: frames per second, Matplotlib blocks for 1/fps seconds after display, defaults to None
+    :type fps: float, optional
 
     :param fig: Matplotlib figure handle to display image on, defaults to new figure
     :type fig: int, optional
@@ -129,10 +135,10 @@ def idisp(im,
     :param extent: extent of the image in user units [xmin, xmax, ymin, ymax]
     :type extent: array_like(4), optional
 
+    :param coordformat: format coordinates and pixel values for the figure window toolbar
+    :type coordformat: callable returning string
     :param savefigname: if not None, save figure as savefigname (default eps)
     :type savefigname: str, optional
-    :param colororder: color order, used for interactive value picker only, defaults to "RGB"
-    :type colororder: str
     :param kwargs: additional options passed through to :func:`matplotlib.pyplot.imshow`.
 
     :return: Matplotlib figure handle and axes handle
@@ -157,6 +163,19 @@ def idisp(im,
     random     random values
     =========  ===============================================
 
+    The argument ``block`` has the following functions
+
+    ``block``            Action after display
+    ===================  ==================================================================================
+    ``False`` (default)  Call ``plt.show(block=False)``, don't block
+    ``True``             Call ``plt.show(block=True)``, block
+    ``None``             Don't call ``plt.show()``, don't block, in Jupyter subsequents plots will be added
+    t:float              Block for set time, calls ``plt.pause(t)``
+    ===================  ==================================================================================
+
+    The ``coordformat`` function is called with (u, v) coordinates and the image is in the variable ``im`` which 
+    is in scope, but not passed, and is an ndarray(H,W) or ndarray(H,W,P).
+
     Example::
 
         >>> from machinevisiontoolbox import iread, idisp
@@ -192,6 +211,9 @@ def idisp(im,
         axes = False
         frame = False
 
+    if fps is not None:
+        block = 1 / fps
+
     # if we are running in a Jupyter notebook, print to matplotlib,
     # otherwise print to opencv imshow/new window. This is done because
     # cv.imshow does not play nicely with .ipynb
@@ -232,11 +254,12 @@ def idisp(im,
                         except:
                             pass
 
-                        if isinstance(block, bool):
-                            plt.show(block=block)
-                        else:
-                            plt.pause(block)
-                        return
+                if block is None:
+                    pass
+                elif isinstance(block, bool):
+                    plt.show(block=block)
+                else:
+                    plt.pause(block)
 
             if fig is not None:
                 # make this figure the current one
@@ -451,7 +474,11 @@ def idisp(im,
         # display the image
         if len(im.shape) == 3:
             # reverse the color planes if it's color
-            h = ax.imshow(im[:, :, ::-1] if bgr else im, norm=norm, cmap=cmap, **options)
+            if colororder not in ("RGB", "BGR"):
+                raise ValueError('unknown colororder ', colororder)
+            if colororder == "BGR":
+                im = im[:, :, ::-1]
+            h = ax.imshow(im, norm=norm, cmap=cmap, **options)
         else:
             if norm is None:
                 # exclude NaN values
@@ -549,7 +576,9 @@ def idisp(im,
                     return f"({u}, {v}): {val} {x.dtype}"
                 else:
                     # color image
-                    x = im[v, u, :]
+                    x = im[v, u, :] # in RGB order
+                    if colororder == "BGR":
+                        x = x[::-1]
                     if np.issubdtype(x.dtype, np.integer):
                         val = [f"{_:d}" for _ in x]
                     elif np.issubdtype(x.dtype, np.floating):
@@ -561,7 +590,10 @@ def idisp(im,
             except IndexError:
                 return ""
 
-        ax.format_coord = format_coord
+        if coordformat is None:
+            ax.format_coord = format_coord
+        else:
+            ax.format_coord = coordformat
 
         # don't display data
         h.format_cursor_data = lambda x: ""
@@ -575,18 +607,29 @@ def idisp(im,
         return h
     else:
         ## display using OpenCV
+        global __last_windowname
+
+        if reuse:
+            if __last_windowname is not None:
+                title = __last_windowname
+            else:
+                title = 'idisp'
+        else:
+            __last_windowname = title
 
         cv.namedWindow(title, cv.WINDOW_AUTOSIZE)
         cv.imshow(title, im)  # make sure BGR format image
 
-        if block:
+        if block is True:
             while True:
                 k = cv.waitKey(delay=0)  # wait forever for keystroke
                 if k == ord('q'):
                     cv.destroyWindow(title)
+                    cv.waitKey(1)
                     break
+        elif isinstance(block, (int, float)):
+            cv.waitKey(round(block * 1000))
             
-
         # TODO fig, ax equivalent for OpenCV? how to print/plot to the same
         # window/set of axes?
         fig = None
@@ -618,12 +661,12 @@ def _isnotebook():
         return False      # Probably standard Python interpreter
 
 
-def iread(filename, *args, verbose=True, **kwargs):
+def iread(filename, *args, **kwargs):
     r"""
     Read image from file or URL
 
-    :param file: file name or URL
-    :type file: str
+    :param filename: file name or URL
+    :type filename: str
     :param kwargs: key word arguments passed to :func:`convert`
     :return: image and filename
     :rtype: tuple (ndarray, str) or list of tuples, image is
@@ -728,7 +771,7 @@ def iread(filename, *args, verbose=True, **kwargs):
         raise ValueError(filename, 'invalid filename')
 
 
-def convert(image, mono=False, gray=False, grey=False, rgb=True, dtype=None, gamma=None, alpha=False, reduce=None, roi=None, maxintval=None):
+def convert(image, mono=False, gray=False, grey=False, rgb=True, dtype=None, gamma=None, alpha=False, reduce=None, roi=None, maxintval=None, copy=False):
     """
     Convert image
 
@@ -740,6 +783,8 @@ def convert(image, mono=False, gray=False, grey=False, rgb=True, dtype=None, gam
     :param dtype: a NumPy dtype string such as ``"uint8"``, ``"int16"``, ``"float32"`` or
         a NumPy type like ``np.uint8``.
     :type dtype: str
+    :param rgb: force color image to RGB order, otherwise BGR
+    :type rgb: bool, optional
     :param gamma: gamma decoding, either the exponent of "sRGB"
     :type gamma: float or str
     :param alpha: allow alpha plane, default False
@@ -750,6 +795,8 @@ def convert(image, mono=False, gray=False, grey=False, rgb=True, dtype=None, gam
     :type roi: array_like(4)
     :param maxintval: maximum integer value to be used for scaling
     :type maxintval: int
+    :param copy: guarantee that returned image is a copy of the input image, defaults to False
+    :type copy: bool
     :return: converted image
     :rtype: ndarray(H,W) or ndarray(H,W,N)
 
@@ -762,6 +809,11 @@ def convert(image, mono=False, gray=False, grey=False, rgb=True, dtype=None, gam
 
     Gamma decoding specified by ``gamma`` can be applied to float or int
     type images.
+
+    The ``grey``/``gray`` option converts a color image to greyscale and is ignored if the
+    image is already greyscale.  Note that this conversions requires knowledge of the color
+    plane order specified by ``colororder``.  The planes are inverted by ``invertplanes`` before
+    this step.
     """
     if grey:
         warnings.warn("grey option to Image.Read/iread is deprecated, use mono instead",
@@ -769,15 +821,24 @@ def convert(image, mono=False, gray=False, grey=False, rgb=True, dtype=None, gam
     if gray:
         warnings.warn("gray option to Image.Read/iread is deprecated, use mono instead",
         DeprecationWarning)
-    mono = mono or gray or grey
-    if mono and len(image.shape) > 2:
-        image = colorspace_convert(image, 'rgb', 'grey')
+    image_original = image
 
-    if image.ndim == 3 and image.shape[2] >= 3:
+    if image.ndim == 3 and image.shape[2] in (3, 4):
+        # is color image RGB, RGBA, BGR, BGRA
         if not alpha:
+            # optionally remove the alpha plane
             image = image[:, :, :3]
         if rgb:
-            image = np.copy(image[:, :, ::-1])  # put in RGB color order
+            # optionally invert the color planes
+            image = np.copy(image[:, :, ::-1])  # reverse the planes
+            # np.copy() is required to make torchvision happy
+            colororder = "RGB"
+        else:
+            colororder = "BGR"
+
+    mono = mono or gray or grey
+    if mono and len(image.shape) == 3:
+        image = colorspace_convert(image, colororder, 'grey')
 
     dtype_alias = {
         'int':    'uint8',
@@ -817,17 +878,20 @@ def convert(image, mono=False, gray=False, grey=False, rgb=True, dtype=None, gam
     if gamma is not None:
         image = gamma_decode(image, gamma)
 
+    if image is not image_original and copy:
+        image = image.copy()
+
     return image
 
 
-def iwrite(im, filename, bgr=False, **kwargs):
+def iwrite(im, filename, colororder="RGB", **kwargs):
     """          
     Write NumPy array to an image file
 
     :param filename: filename to write to
     :type filename: string
-    :param bgr: image is in BGR (native OpenCV color) order, defaults to False
-    :type bgr: bool, optional
+    :param colororder: color order, defaults to "RGB"
+    :type colororder: str
     :param kwargs: additional arguments, see ImwriteFlags
     :return: successful write
     :rtype: bool
@@ -851,12 +915,11 @@ def iwrite(im, filename, bgr=False, **kwargs):
 
     :seealso: :func:`iread` `cv2.imwrite <https://docs.opencv.org/4.x/d4/da8/group__imgcodecs.html#gabbc7ef1aa2edfaa87772f1202d67e0ce>`_ 
     """
-    if bgr:
-        # write BGR format image directly
-        return cv.imwrite(filename, im, **kwargs)
-    elif im.ndim > 2:
-        # otherwise, if color, flip the planes
+    if im.ndim > 2 and colororder == "RGB":
+        # put image into OpenCV BGR order for writing
         return cv.imwrite(filename, im[:, :, ::-1], **kwargs)
+    else:
+        return cv.imwrite(filename, im, **kwargs)
 
 def pickpoints(self, n=None, matplotlib=True):
     """
@@ -909,10 +972,34 @@ if __name__ == "__main__":
     # im = iread(filename)
     # print(im[0])
 
-    im = np.zeros((4,4))
-    im[:,0] = 0.2
-    im[:,3] = -0.4
-    idisp(im, colormap='signed', block=True)
+    from machinevisiontoolbox import VideoCamera, Image
+    from machinevisiontoolbox.base import idisp
+    import numpy as np
+    c = VideoCamera(0, rgb=False)
+    x = c.grab()
+
+    x.disp(block=True)
+    #x.disp(block=True)
+    # idisp(x.image, colororder="BGR", block=True)
+
+    # a = np.eye(2)
+    # b = np.dstack([a, a*0, a*0])
+    # # idisp(b)
+    # # idisp(b, colororder="RGB", title="RGB")
+    # # idisp(b, colororder="BGR", title="BGR", block=True)
+
+    # x = Image(b)
+    # y = Image(b, colororder="BGR")
+    # x.disp()
+    # y.disp(block=True)
+
+
+
+
+    # im = np.zeros((4,4))
+    # im[:,0] = 0.2
+    # im[:,3] = -0.4
+    # idisp(im, colormap='signed', block=True)
     # for i in range(100):
     #     im[:,i] = i - 40
     # idisp(im, matplotlib=True, title='default')
@@ -931,5 +1018,5 @@ if __name__ == "__main__":
     # idisp(im, matplotlib=True, block=False, colormap='grey', xydata=np.r_[10,20,30,40], title='grey')
     # idisp(im, matplotlib=True, block=True, colormap='grey', ynormal=True, title='grey')
 
-    im, file = iread('street.png', dtype='float')
-    idisp(im, title='Boo!', block=True)
+    # im, file = iread('street.png', dtype='float')
+    # idisp(im, title='Boo!', block=True)
